{*************************************************************}
{            DiskInfo component for Delphi 32                 }
{ Version:   1.2                                              }
{ Author:    Aleksey Kuznetsov                                }
{ E-Mail:    info@utilmind.com                                }
{ Home Page: http://www.utilmind.com                          }
{ Created:   May, 4, 1999                                     }
{ Modified:  May, 20, 1999                                    }
{ Legal:     Copyright (c) 1999, UtilMind Solutions           }
{ Modified By: Cayetano Gmez ( cayetano@iname.com )          }
{*************************************************************}
{  TDiskInfo (English):                                       }
{ Component determines the information about specified local  }
{ or a network disk - Serial number, Volume label, type of    }
{ file system, type of a disk, size of free and engaged space }
{*************************************************************}
{  TDiskInfo (Russian):                                       }
{        }
{   -  ,  ,      }
{ ,  ,                }
{ .                                               }
{*************************************************************}
{ PROPERTIES:                                                 }
{   Disk: Char - Drive letter                                 }
{                                                             }
{ READ-ONLY PROPERTIES (results)                              }
{   SerialNumberStr: String                                   }
{   SerialNumber: LongInt                                     }
{   VolumeLabel: String                                       }
{   FileSystem: String                                        }
{   DriveType: TDriveType                                     }
{   DiskSize: Int64                                           }
{   DiskFree: Int64                                           }
{   DiskFreeStr: String                                       }
{   DiskSizeStr: String                                       }
{*************************************************************}
{ Please see demo program for more information.               }
{*************************************************************}
{                     IMPORTANT NOTE:                         }
{ This software is provided 'as-is', without any express or   }
{ implied warranty. In no event will the author be held       }
{ liable for any damages arising from the use of this         }
{ software.                                                   }
{ Permission is granted to anyone to use this software for    }
{ any purpose, including commercial applications, and to      }
{ alter it and redistribute it freely, subject to the         }
{ following restrictions:                                     }
{ 1. The origin of this software must not be misrepresented,  }
{    you must not claim that you wrote the original software. }
{    If you use this software in a product, an acknowledgment }
{    in the product documentation would be appreciated but is }
{    not required.                                            }
{ 2. Altered source versions must be plainly marked as such,  }
{    and must not be misrepresented as being the original     }
{    software.                                                }
{ 3. This notice may not be removed or altered from any       }
{    source distribution.                                     }
{*************************************************************}

Unit DiskInfo;

Interface

Uses
    Windows, Classes, SysUtils;

Type
    TDriveType = ( dtUnknown, dtNoDrive, dtFloppy, dtFixed, dtNetwork,
        dtCDROM, dtRAM );

    TDiskInfo = Class( TComponent )
    Private
        FDisk : Char;
        FSerialNumberStr : String;
        FSerialNumber : LongInt;
        FVolumeLabel : String;
        FFileSystem : String;
        FDriveType : TDriveType;
        FDiskSize : Int64;
        FDiskFree : Int64;
        FDIskSizeStr : String;
        FDiskFreeStr : String;

        Procedure SetDisk( Value : Char );
        Procedure SetNothing( Value : String );
        Procedure SetNothingLong( Value : LongInt );
        Procedure SetNothingINT64( Value : Int64 );
        Procedure SetNothingDT( Value : TDriveType );

        Function GetStrCant( nOrg : Int64 ) : String;

    Protected
    Public
        Constructor Create( aOwner : TComponent ); Override;

    Published

        Property Disk : Char Read FDisk Write SetDisk;
        Property SerialNumberStr : String Read FSerialNumberStr Write
            SetNothing;
        Property SerialNumber : LongInt Read FSerialNumber Write
            SetNothingLong;
        Property VolumeLabel : String Read FVolumeLabel Write SetNothing;
        Property FileSystem : String Read FFileSystem Write SetNothing;
        Property DriveType : TDriveType Read FDriveType Write SetNothingDT;
        Property DiskSize : Int64 Read FDiskSize Write SetNothingInt64;
        Property DiskFree : Int64 Read FDiskFree Write SetNothingInt64;
        Property DiskSizeStr : String Read FDiskSizeStr Write SetNothing;
        Property DiskFreeStr : String Read FDiskFreeStr Write SetNothing;
    End;

Procedure Register;

Implementation

Constructor TDiskInfo.Create( aOwner : TComponent );
Begin
    Inherited Create( aOwner );
    FDisk := 'C';
End;

Procedure TDiskInfo.SetDisk( Value : Char );
Var
    VolumeLabel, FileSystem : Array[ 0..$FF ] Of Char;
{$IFDEF Ver120}     // Delphi4 requires DWord !
    SerialNumber, Comps, SysFlags : DWord;
{$ELSE}
    SerialNumber, Comps, SysFlags : Integer;
{$ENDIF}

    Function DecToHex( aValue : LongInt ) : String;
    Var
        w : Array[ 1..2 ] Of Word Absolute aValue;

        Function HexByte( b : Byte ) : String;
        Const
            Hex : Array[ $0..$F ] Of Char = '0123456789ABCDEF';
        Begin
            HexByte := Hex[ b Shr 4 ] + Hex[ b And $F ];
        End;

        Function HexWord( w : Word ) : String;
        Begin
            HexWord := HexByte( Hi( w ) ) + HexByte( Lo( w ) );
        End;

    Begin
        Result := HexWord( w[ 2 ] ) + HexWord( w[ 1 ] );
    End;

Begin
    Value := UpCase( Value );
    If ( Value >= 'A' ) And ( Value <= 'Z' ) Then
    Begin
        FDisk := Value;
        GetVolumeInformation( PChar( Value + ':\' ), VolumeLabel, SizeOf(
            VolumeLabel ),
            @SerialNumber, Comps, SysFlags,
            FileSystem, SizeOf( FileSystem ) );
        FSerialNumber := SerialNumber;
        FSerialNumberStr := DecToHex( SerialNumber );
        Insert( '-', FSerialNumberStr, 5 );
        FVolumeLabel := VolumeLabel;
        FFileSystem := FileSystem;
        FDriveType := TDriveType( GetDriveType( PChar( Value + ':\' ) ) );
        FDiskSize := SysUtils.DiskSize( Byte( Value ) - 64 );
        FDiskFree := SysUtils.DiskFree( Byte( Value ) - 64 );
        FDiskSizeStr := GetStrCant( FDiskSize );
        FDiskFreeStr := GetStrCant( FDiskFree );
    End
End;

Procedure TDiskInfo.SetNothing( Value : String );
Begin               {}
End;

Procedure TDiskInfo.SetNothingLong( Value : LongInt );
Begin               {}
End;

Procedure TDiskInfo.SetNothingInt64( Value : Int64 );
Begin               {}
End;

Procedure TDiskInfo.SetNothingDT( Value : TDriveType );
Begin               {}
End;

Procedure Register;
Begin
    RegisterComponents( 'UtilMind', [ TDiskInfo ] );
End;


Function TDiskInfo.GetStrCant( nOrg : Int64 ) : String;
Var
    cUnit : String;
    nCant : Double;
Begin
    cUnit := ' ';
    nCant := nOrg;  // cast
    If nCant > 1024 Then
    Begin
        nCant := nCant / 1024;
        cUnit := 'K'
    End;

    If nCant > 1024 Then
    Begin
        nCant := nCant / 1024;
        cUnit := 'M'
    End;

    If nCant > 1024 Then
    Begin
        nCant := nCant / 1024;
        cUnit := 'G'
    End;

    If nCant > 1024 Then
    Begin           // :-))
        nCant := nCant / 1024;
        cUnit := 'T'
    End;

    Result := Format( '%3.2f %s', [ nCant, cUnit ] )
End;

End.