unit RRSDateTime;

interface

uses
	Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
	ComCtrls, Commctrl, DB, DBCtrls;

type
	TRRSDateTime = class(TDateTimePicker)
	private
		FLongDateFormat: string;
		FLongTimeFormat: string;
		FShortDateFormat: string;
		FShortTimeFormat: string;
	protected
		procedure CreateWnd; override;
		function GetDateFormat: TDTDateFormat;
		procedure SetDateFormat(Value: TDTDateFormat);

		function GetKind: TDateTimeKind;
		procedure SetKind(Value: TDateTimeKind);

		procedure SetLongDateFormat(const Value: string);
		procedure SetLongTimeFormat(const Value: string);
		procedure SetShortDateFormat(const Value: string);
		procedure SetShortTimeFormat(const Value: string);
		procedure CheckEmptyDate; override;
	public
		constructor Create(AOwner: TComponent); override;
	published
		property DateFormat: TDTDateFormat read GetDateFormat write SetDateFormat;
		property Kind: TDateTimeKind read GetKind write SetKind;
		property LongDateFormat: string read FLongDateFormat write SetLongDateFormat;
		property LongTimeFormat: string read FLongTimeFormat write SetLongTimeFormat;
		property ShortTimeFormat: string read FShortTimeFormat write SetShortTimeFormat;
		property ShortDateFormat: string read FShortDateFormat write SetShortDateFormat;
	end;

type
	TRRSDBDateTime = class(TRRSDateTime)
	private
		FDataLink: TFieldDataLink;
		FReadOnly: Boolean;
		procedure DataChange(Sender: TObject);
		function GetDataField: string;
		function GetDataSource: TDataSource;
		function GetField: TField;
		procedure SetDataField(const Value: string);
		procedure SetDataSource(Value: TDataSource);
		procedure SetReadOnly(Value: Boolean);
		procedure UpdateData(Sender: TObject);
		procedure WMCut(var Message: TMessage); message WM_CUT;
		procedure WMPaste(var Message: TMessage); message WM_PASTE;
		procedure CMExit(var Message: TCMExit); message CM_EXIT;
		procedure CMGetDataLink(var Message: TMessage); message CM_GETDATALINK;
		function GetDate: TDateTime;
		function GetTime: TDateTime;
	protected
		procedure Change; override;
		procedure KeyDown(var Key: Word; Shift: TShiftState); override;
		procedure KeyPress(var Key: Char); override;
		procedure Loaded; override;
		procedure Notification(AComponent: TComponent; Operation: TOperation); override;
	public
		constructor Create(AOwner: TComponent); override;
		destructor Destroy; override;
		function ExecuteAction(Action: TBasicAction): Boolean; override;
		function UpdateAction(Action: TBasicAction): Boolean; override;
		property Field: TField read GetField;
	published
		property DataField: string read GetDataField write SetDataField;
		property DataSource: TDataSource read GetDataSource write SetDataSource;
		property ReadOnly: Boolean read FReadOnly write SetReadOnly default False;
		property Date: TDateTime read GetDate;
		property Time: TDateTime read GetTime;
	end;

procedure Register;

implementation

procedure Register;
begin
	RegisterComponents('RRS', [TRRSDateTime]);
	RegisterComponents('RRS', [TRRSDBDateTime]);
end;

procedure int3; assembler;
asm
//	int 3;
end;

{TRRSDateTime}

constructor TRRSDateTime.Create(AOwner: TComponent);
var
	i: Integer;
	DefaultLCID: LCID;
begin
	inherited Create(AOwner);
	DefaultLCID	:= GetThreadLocale;
	FShortDateFormat := GetLocaleStr(DefaultLCID, LOCALE_SSHORTDATE, 'M/dd/yyyy');
	FLongDateFormat := GetLocaleStr(DefaultLCID, LOCALE_SLONGDATE, 'MMM dd, yyyy');
	FLongTimeFormat := GetLocaleStr(DefaultLCID, LOCALE_STIMEFORMAT, 'hh:mm:ss tt');
	FShortTimeFormat := FLongTimeFormat;
	i := Pos('s', FShortTimeFormat);
	if i > 0 then begin
		if FShortTimeFormat[i - 1] = TimeSeparator then
			Delete(FShortTimeFormat, i - 1, 1);
	end;
	Dec(i);
	while i > 0 do begin
		Delete(FShortTimeFormat, i, 1);
		i := Pos('s', FShortTimeFormat);
	end;
end;

procedure TRRSDateTime.CreateWnd;
begin
	inherited CreateWnd;
	if Kind = dtkTime then begin
		if DateFormat = dfShort then begin
			if FShortTimeFormat > '' then
				SendMessage(Handle, DTM_SETFORMAT, 0, LongInt(PChar(FShortTimeFormat)));
		end else begin
			if FLongTimeFormat > '' then
				SendMessage(Handle, DTM_SETFORMAT, 0, LongInt(PChar(FLongTimeFormat)));
		end;
	end else begin
		if DateFormat = dfShort then begin
			if FShortDateFormat > '' then
				SendMessage(Handle, DTM_SETFORMAT, 0, LongInt(PChar(FShortDateFormat)));
		end else begin
			if FLongDateFormat > '' then
				SendMessage(Handle, DTM_SETFORMAT, 0, LongInt(PChar(FLongDateFormat)));
		end;
	end;

end;

function TRRSDateTime.GetDateFormat: TDTDateFormat;
begin
	Result := inherited DateFormat;
end;

procedure TRRSDateTime.SetDateFormat(Value: TDTDateFormat);
begin
	inherited DateFormat := Value;
end;

function TRRSDateTime.GetKind: TDateTimeKind;
begin
	Result := inherited Kind;
end;

procedure TRRSDateTime.SetKind(Value: TDateTimeKind);
begin
	if inherited Kind = Value then
		Exit;
	inherited Kind := Value;
	if (Kind = dtkTime) then
		DateMode := dmUpDown;
	RecreateWnd;
end;

procedure TRRSDateTime.SetLongTimeFormat(const Value: string);
begin
	if FLongTimeFormat = Value then
		Exit;
	FLongTimeFormat := Value;
	RecreateWnd;
end;

procedure TRRSDateTime.SetLongDateFormat(const Value: string);
begin
	if FLongDateFormat = Value then
		Exit;
	FLongDateFormat := Value;
	RecreateWnd;
end;

procedure TRRSDateTime.SetShortDateFormat(const Value: string);
begin
	if FShortDateFormat = Value then
		Exit;
	FShortDateFormat := Value;
	RecreateWnd;
end;

procedure TRRSDateTime.SetShortTimeFormat(const Value: string);
begin
	if FShortTimeFormat = Value then
		Exit;
	FShortTimeFormat := Value;
	RecreateWnd;
end;

procedure TRRSDateTime.CheckEmptyDate;
begin
	// do nothing
end;


{TRRSDBDateTime}

constructor TRRSDBDateTime.Create(AOwner: TComponent);
begin
	inherited Create(AOwner);
	FReadOnly := False;
	FDataLink := TFieldDataLink.Create;
	FDataLink.Control := Self;
	FDataLink.OnDataChange := DataChange;
//	FDataLink.OnEditingChange := EditingChange;
	FDataLink.OnUpdateData := UpdateData;
end;

destructor TRRSDBDateTime.Destroy;
begin
	FDataLink.Free;
	FDataLink := nil;
	inherited Destroy;
end;

procedure TRRSDBDateTime.Loaded;
begin
	inherited Loaded;
	if (csDesigning in ComponentState) then DataChange(Self);
end;

procedure TRRSDBDateTime.Notification(AComponent: TComponent;
	Operation: TOperation);
begin
	inherited Notification(AComponent, Operation);
	if (Operation = opRemove) and (FDataLink <> nil) and
		(AComponent = DataSource) then DataSource := nil;
end;

procedure TRRSDBDateTime.KeyDown(var Key: Word; Shift: TShiftState);
begin
  inherited KeyDown(Key, Shift);
  if (Key = VK_DELETE) or ((Key = VK_INSERT) and (ssShift in Shift)) then
    FDataLink.Edit;
end;

procedure TRRSDBDateTime.KeyPress(var Key: Char);
begin
int3;
	inherited KeyPress(Key);
  if (Key in [#32..#255]) and (FDataLink.Field <> nil) and
    not FDataLink.Field.IsValidChar(Key) then
	begin
		MessageBeep(0);
		Key := #0;
	end;
	case Key of
		^H, ^V, ^X, #32..#255:
			FDataLink.Edit;
		#27:
			begin
				FDataLink.Reset;
				Key := #0;
			end;
	end;
end;

procedure TRRSDBDateTime.Change;
begin
int3;
	FDataLink.Modified;
	inherited Change;
end;

function TRRSDBDateTime.GetDataSource: TDataSource;
begin
	Result := FDataLink.DataSource;
end;

procedure TRRSDBDateTime.SetDataSource(Value: TDataSource);
begin
	if not (FDataLink.DataSourceFixed and (csLoading in ComponentState)) then
		FDataLink.DataSource := Value;
	if Value <> nil then Value.FreeNotification(Self);
end;

function TRRSDBDateTime.GetDataField: string;
begin
	Result := FDataLink.FieldName;
end;

procedure TRRSDBDateTime.SetDataField(const Value: string);
begin
	FDataLink.FieldName := Value;
end;

procedure TRRSDBDateTime.SetReadOnly(Value: Boolean);
begin
	FReadOnly := Value;
	FDataLink.ReadOnly := Value;
end;

function TRRSDBDateTime.GetField: TField;
begin
	Result := FDataLink.Field;
end;

procedure TRRSDBDateTime.DataChange(Sender: TObject);
begin
	if FDataLink.Field = nil then begin
//		Checked := False;
		DateTime := Now;
//		RecreateWnd;
		Exit;
	end;
	with FDataLink.Field do
		if IsNull then begin
//			Checked := False;
			DateTime := Now;
//			RecreateWnd;
		end else begin
//			Checked := True;
			DateTime := AsDateTime;
//			RecreateWnd;
		end;
{
	if FDataLink.Editing and FDataLink.FModified then
		Modified := True;
}
end;

procedure TRRSDBDateTime.UpdateData(Sender: TObject);
begin
int3;
	if not FDataLink.Editing then
		FDataLink.Edit;
//	if not Checked then begin
//		FDataLink.Field.Value := Null;
//	end else begin
		FDataLink.Field.AsDateTime := DateTime;
//	end;
end;

procedure TRRSDBDateTime.WMPaste(var Message: TMessage);
begin
	FDataLink.Edit;
	inherited;
end;

procedure TRRSDBDateTime.WMCut(var Message: TMessage);
begin
	FDataLink.Edit;
	inherited;
end;

procedure TRRSDBDateTime.CMExit(var Message: TCMExit);
begin
int3;
	try
		FDataLink.UpdateRecord;
	except
		SetFocus;
		raise;
	end;
	inherited;
end;

procedure TRRSDBDateTime.CMGetDataLink(var Message: TMessage);
begin
	Message.Result := Integer(FDataLink);
end;

function TRRSDBDateTime.ExecuteAction(Action: TBasicAction): Boolean;
begin
  Result := inherited ExecuteAction(Action) or (FDataLink <> nil) and
    FDataLink.ExecuteAction(Action);
end;

function TRRSDBDateTime.UpdateAction(Action: TBasicAction): Boolean;
begin
	Result := inherited UpdateAction(Action) or (FDataLink <> nil) and
		FDataLink.UpdateAction(Action);
end;

function TRRSDBDateTime.GetDate: TDateTime;
begin
	Result := inherited Date;
end;

function TRRSDBDateTime.GetTime: TDateTime;
begin
	Result := inherited Time;
end;

end.
