unit jvGridFilter;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, Grids;

type
  TjvGridFilter = class(TComponent)
  private
    FGrid: TStringGrid;
    procedure ApplyFilter;

    function parseFilter(Afilter: string): boolean;
    procedure SetGrid(const Value: TStringGrid);
    { Private declarations }
  protected
    { Protected declarations }
  public
    { Public declarations }
    procedure Filter(AFilter: string);
    procedure ShowRows;
  published
    { Published declarations }
    property Grid:TStringGrid read FGrid write SetGrid;
  end;

procedure Register;

implementation

Type
 TGridFilterFunc=function(FieldValue,FilterValue:string):boolean;

 TGridFieldFilter=record
  FilterFunc: TGridFilterFunc;
  FilterField: integer;
  FilterValue:string;
  end;

 TGridRowFilter=record
  FilterCount:integer;
  Filters: array[0..9] of TGridFieldFilter;
  end;

var
 GridRowFilter: TGridRowFilter;


procedure Register;
begin
  RegisterComponents('Jans 2', [TjvGridFilter]);
end;


// Grid filter functions

function filterEQ(FieldValue,FilterValue:string):boolean;
begin
 result:=FieldValue=FilterValue;
end;

function filterNE(FieldValue,FilterValue:string):boolean;
begin
 result:=FieldValue<>FilterValue;
end;

function filterGT(FieldValue,FilterValue:string):boolean;
begin
 result:=FieldValue>FilterValue;
end;

function filterLT(FieldValue,FilterValue:string):boolean;
begin
 result:=FieldValue<FilterValue;
end;

function filterLIKE(FieldValue,FilterValue:string):boolean;
begin
 result:=pos(lowercase(FilterValue),lowercase(FieldValue))>0;
end;

function TjvGridFilter.parseFilter(Afilter:string):boolean;
var op,s:string;
    f : TGridFilterFunc;
    fieldnr,i,p:integer;
    Fieldname,Filtervalue:string;
begin
 result:=false;
 GridRowFilter.FilterCount :=0;
 s:=trim(Afilter);
 if s='' then exit;
 // parse field name
 repeat
 p:=pos('[',s);

 if p=0 then exit;
 s:=copy(s,p+1,length(s));
 p:=pos(']',s);
 if p=0 then exit;
 Fieldname:=copy(s,1,p-1);
 s:=trim(copy(s,p+1,length(s)));
 if FieldName='' then exit;
 // find fieldnumber
 FieldNr:=0;
 for i:=1 to Grid.colcount-1 do
  if Grid.cells[i,0]=Fieldname then begin
   Fieldnr:=i;
   Break;
   end;
 if fieldnr=0 then exit;
 // we have the field number, now check operand
 p:=pos('"',s); // " marks the beginning of the filter value
 if p=0 then exit;
 op:=lowercase(trim(copy(s,1,p-1)));
 s:=copy(s,p+1,length(s));
 p:=pos('"',s); // find the end of the filtervalue
 if p=0 then exit;
 FilterValue:=copy(s,1,p-1);
 s:=trim(copy(s,p+1,length(s)));
 if op='=' then
  f :=filterEQ
 else if op='<>' then
  f :=filterNE
 else if op='>' then
  f :=filterGT
 else if op='<' then
  f :=filterLT
 else if op='like' then
  f :=filterLIKE
  else exit;
 inc(GridRowFilter.FilterCount);
 if (GridRowFilter.FilterCount>9) then begin
  showmessage('Filter too complex');
  GridRowFilter.FilterCount:=0;
  exit;
  end;
 GridrowFilter.Filters[Gridrowfilter.FilterCount-1].FilterFunc :=f;
 GridrowFilter.Filters[Gridrowfilter.FilterCount-1].FilterField :=FieldNr;
 GridrowFilter.Filters[Gridrowfilter.FilterCount-1].FilterValue :=FilterValue;
 until s='';
 result:=true;
end;

procedure TjvGridFilter.ApplyFilter;
var arow,fi,fc,Filterfield:integer;
    FieldValue,FilterValue:string;
    fn:TGridFilterFunc;
    CanHide:boolean;
begin
 if GridRowFilter.FilterCount =0 then exit;
 fc:=GridRowFilter.FilterCount;
  for arow:=1 to Grid.rowcount-1 do begin
   CanHide:=false;
   for fi:=0 to fc-1 do begin
    fn:= GridRowFilter.Filters [fi].FilterFunc ;
    FilterValue:=GridRowFilter.Filters [fi].FilterValue ;
    filterField:=GridRowFilter.Filters [fi].FilterField ;
    FieldValue:=Grid.cells[FilterField,arow];
    if not fn(FieldValue,FilterValue) then begin
     CanHide:=true;
     break;
     end;
    end;
   if CanHide then Grid.rowheights[arow]:=0;
   end;
end;

procedure TjvGridFilter.Filter(AFilter: string);
begin
 if assigned(FGrid) then
  if parseFilter(AFilter) then
   ApplyFilter;
end;



procedure TjvGridFilter.SetGrid(const Value: TStringGrid);
begin
  FGrid := Value;
end;

procedure TjvGridFilter.ShowRows;
var arow:integer;
begin
 for arow:=0 to Grid.rowcount-1 do
  Grid.rowheights[arow]:=grid.defaultrowheight;
end;

end.
